import math
from typing import Iterable


def percentile(N: Iterable, percent: int):
    """
    Find the percentile of a list of values.
    Stolen from http://code.activestate.com/recipes/511478-finding-the-percentile-of-the-values/
    """
    if not N:
        raise ValueError('N must be non-empty iterable')

    if not (0 < percent < 100 and type(percent) == int):
        raise ValueError('percent parameter must be integer from 0 to 100')

    N = sorted(N)

    k = (len(N) - 1) * percent / 100
    prev_index = math.floor(k)
    next_index = math.ceil(k)

    if prev_index == next_index:
        return N[int(k)]

    d0 = N[prev_index] * (next_index - k)
    d1 = N[next_index] * (k - prev_index)

    return d0 + d1



from functools import partial

import pytest

from percentiles import percentile


percentile_75 = partial(percentile, percent=75)


@pytest.mark.parametrize('values, expected', (
    ([100, 200, 400], 300),
    ([1, 7, 5, 3], 5.5),
    ([1.1, 700.5, 2.3], 351.4),
    ([1.1, 700.5, 2.3, 0.1, 4, 6, 90, 24, 33.45], 33.45),
    ([100], 100),
    ([100, 100], 100),
))
def test_75_percentile_function(values, expected):

    assert percentile_75(values) == expected


@pytest.mark.parametrize('value', (0.5, 142, -5))
def test_raises_when_percent_is_incorrect(value):
    with pytest.raises(ValueError):
        percentile([100], value)


def test_raises_when_on_empty_input():
    with pytest.raises(ValueError):
        percentile([], 5)



from setuptools import setup

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='percentiles',
    use_scm_version=True,
    description="Get percentile of sequence without numpy",
    keywords=['percentile', 'statistics'],
    url="https://github.com/heaviss/percentiles",
    author="Vladimir Seregin",
    author_email="31631@rambler.ru",
    license="MIT",
    packages=["percentiles"],
    install_requires=[],
    setup_requires=[
        'setuptools_scm',
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
)
